import numpy as np
from scipy.special import binom

delta = 1.0
omega = 1.0
lambda_min = -delta/2.0

def lnqp(q, p, length):
    """ apply the Liouville operator to (q, p), i.e. (iL)^n q and (iL)^n p """

    sqq = [1, 0, -1, 0]
    sqp = [0, 1, 0, -1]
    spq = [0, -1, 0, 1]
    spp = [1, 0, -1, 0]
    lnq = np.zeros(length, dtype=complex)
    lnp = np.zeros(length, dtype=complex)
    for k in range(length):
        cyc = k % 4
        lnq[k] = q * sqq[cyc] + p * sqp[cyc]
        lnp[k] = q * spq[cyc] + p * spp[cyc]
    # scaling the powers of the Liouville operator
    a = 1./delta
    if np.isclose(a, 1.0, atol=1e-08):
        return (lnq, lnp)
    else:
        lnqsc = np.zeros(length, dtype=complex)
        lnpsc = np.zeros(length, dtype=complex)
        b = -a*(lambda_min + delta/2.0)
        if np.isclose(b, 0.0, atol=1e-08):
            for n in range(length):
                lnqsc[n] = lnq[n] * a**n
                lnpsc[n] = lnp[n] * a**n
        else:
            for n in range(length):
                for k in range(n+1):
                    bnk = binom(n, k)
                    lnqsc[n] = lnqsc[n] + bnk * a**k * b**(n-k) * lnq[k]
                    lnpsc[n] = lnpsc[n] + bnk * a**k * b**(n-k) * lnp[k]
        return (lnqsc, lnpsc)

def force(q):
    return -q

def energy(q, p):
    return 0.5*(p*p + q*q)

def solution(q0, p0, time):
    q_ref = q0*np.exp(1.0j*omega*time)
    p_ref = q0*(1.0j*omega)*np.exp(1.0j*omega*time)
    return (q_ref, p_ref)
